from django.db import models
from django.utils import timezone


class TimeTrackedModel(models.Model):
    created_at = models.DateTimeField(editable=False)
    updated_at = models.DateTimeField()

    def save(self, *args, **kwargs):
        ''' On save, update timestamps '''
        if not self.id:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()
        return super(TimeTrackedModel, self).save(*args, **kwargs)

    class Meta:
        abstract = True


class SurveyJob(TimeTrackedModel):
    source_type = models.CharField(max_length=256)

    # The start time of the query used to replicate
    replication_started_at = models.DateTimeField()

    # The end time of the query used to replicate
    replication_ended_at = models.DateTimeField()

    # The start time of the job
    start_time = models.DateTimeField()

    # The end time of the job
    end_time = models.DateTimeField()

    class Meta:
        db_table = "survey_jobs"


# This model is still has a prototypical status, but I needed something to
# test with and it's at least in the right ballpark
class Batch(TimeTrackedModel):
    survey_job = models.ForeignKey(SurveyJob)
    source_type = models.CharField(max_length=256)
    size_in_bytes = models.IntegerField()
    download_url = models.CharField(max_length=256)
    raw_format = models.CharField(max_length=256)
    processed_format = models.CharField(max_length=256)
    pipeline_required = models.CharField(max_length=256)
    accession_code = models.CharField(max_length=32)

    # This field where denote where in our system the file can be found
    internal_location = models.CharField(max_length=256)

    # This will at some point be a meaningful integer->organism lookup thing
    organism = models.IntegerField()

    STATUSES = (
        ("NEW", "New"),
        ("DOWNLOADING", "Downloading"),
        ("DOWNLOADED", "Downloaded"),
        ("PROCESSING", "Processing"),
        ("PROCESSED", "Proccessed"),
    )
    status = models.CharField(max_length=10, choices=STATUSES)

    class Meta:
        db_table = "batches"


# This table is used for tacking fields onto a Batch record that would
# be sparsly populated if it was its own column.
# I.e. one source may have an extra field or two that are worth tracking
# but are specific to that source.
class BatchKeyValue(TimeTrackedModel):
    batch_id = models.ForeignKey(Batch, on_delete=models.CASCADE)
    key = models.CharField(max_length=256)
    value = models.CharField(max_length=256)

    class Meta:
        db_table = "batch_key_values"


class ProcessorJob(TimeTrackedModel):
    batch_id = models.ForeignKey(Batch, on_delete=models.CASCADE)
    pipeline_applied = models.CharField(max_length=256)
    start_time = models.DateTimeField()
    end_time = models.DateTimeField()
    success = models.NullBooleanField()
    num_retries = models.IntegerField()

    # This point of this field is to identify what worker ran the job.
    # A few fields may actually be required or something other than just an id.
    worker_id = models.CharField(max_length=256)

    class Meta:
        db_table = "processor_jobs"


class DownloaderJob(TimeTrackedModel):
    batch_id = models.ForeignKey(Batch, on_delete=models.CASCADE)
    start_time = models.DateTimeField()
    end_time = models.DateTimeField()
    success = models.NullBooleanField()
    num_retries = models.IntegerField()
    worker_id = models.CharField(max_length=256)

    class Meta:
        db_table = "downloader_jobs"



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-17 17:16
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Batch',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(editable=False)),
                ('updated_at', models.DateTimeField()),
                ('source_type', models.CharField(max_length=256)),
                ('size_in_bytes', models.IntegerField()),
                ('download_url', models.CharField(max_length=256)),
                ('raw_format', models.CharField(max_length=256)),
                ('processed_format', models.CharField(max_length=256)),
                ('pipeline_required', models.CharField(max_length=256)),
                ('accession_code', models.CharField(max_length=32)),
                ('internal_location', models.CharField(max_length=256)),
                ('organism', models.IntegerField()),
                ('status', models.CharField(choices=[(b'NEW', b'New'), (b'DOWNLOADING', b'Downloading'), (b'DOWNLOADED', b'Downloaded'), (b'PROCESSING', b'Processing'), (b'PROCESSED', b'Proccessed')], max_length=10)),
            ],
            options={
                'db_table': 'batches',
            },
        ),
        migrations.CreateModel(
            name='BatchKeyValue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(editable=False)),
                ('updated_at', models.DateTimeField()),
                ('key', models.CharField(max_length=256)),
                ('value', models.CharField(max_length=256)),
                ('batch_id', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bioinformatics_mill_models.Batch')),
            ],
            options={
                'db_table': 'batch_key_values',
            },
        ),
        migrations.CreateModel(
            name='DownloaderJob',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(editable=False)),
                ('updated_at', models.DateTimeField()),
                ('start_time', models.DateTimeField()),
                ('end_time', models.DateTimeField()),
                ('success', models.NullBooleanField()),
                ('num_retries', models.IntegerField()),
                ('worker_id', models.CharField(max_length=256)),
                ('batch_id', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bioinformatics_mill_models.Batch')),
            ],
            options={
                'db_table': 'downloader_jobs',
            },
        ),
        migrations.CreateModel(
            name='ProcessorJob',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(editable=False)),
                ('updated_at', models.DateTimeField()),
                ('pipeline_applied', models.CharField(max_length=256)),
                ('start_time', models.DateTimeField()),
                ('end_time', models.DateTimeField()),
                ('success', models.NullBooleanField()),
                ('num_retries', models.IntegerField()),
                ('worker_id', models.CharField(max_length=256)),
                ('batch_id', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bioinformatics_mill_models.Batch')),
            ],
            options={
                'db_table': 'processor_jobs',
            },
        ),
        migrations.CreateModel(
            name='SurveyJob',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(editable=False)),
                ('updated_at', models.DateTimeField()),
                ('source_type', models.CharField(max_length=256)),
                ('replication_started_at', models.DateTimeField()),
                ('replication_ended_at', models.DateTimeField()),
                ('start_time', models.DateTimeField()),
                ('end_time', models.DateTimeField()),
            ],
            options={
                'db_table': 'survey_jobs',
            },
        ),
        migrations.AddField(
            model_name='batch',
            name='survey_job',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bioinformatics_mill_models.SurveyJob'),
        ),
    ]



import os
from setuptools import find_packages, setup

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='bioinformatics-mill-models',
    version='0.1.1',
    packages=find_packages(),
    include_package_data=True,
    license='BSD License',
    description='An abstracted database model for the Bioinformatics Mill project',
    url='https://www.greenelab.com',
    author='Kurt Wheeler',
    author_email='kurt.wheeler91@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
    ],
)
